package com.malk.yangu.server.model;

import com.malk.base.BaseDto;
import com.malk.server.common.McException;
import com.malk.server.common.McREnum;
import com.malk.utils.UtilMap;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;
import org.apache.commons.lang3.StringUtils;

import java.math.BigDecimal;
import java.util.Map;

@Data
@Builder
@AllArgsConstructor
@NoArgsConstructor
public class McInvoiceDto extends BaseDto {


    /**************** 发票 ****************/

    /**
     * 发票名称
     */
    private String name;

    // 兼容: 不为空
    public String getName() {
        if (StringUtils.isBlank(name)) {
            return kindName;
        }
        return name;
    }

    /**
     * 发票类型
     */
    private String kindName;

    /**
     * 发票类型 [编码]
     */
    private int kind;

    /**
     * 发票代码
     */
    private String code;

    /**
     * 发票号码
     */
    private String serial;

    /**
     * 开票日期 [yyyy-MM-dd]
     */
    private String date;

    /**
     * 校验码
     */
    private String checkCode;

    /**
     * 价税合计
     */
    private BigDecimal amount;

    /**
     * 不含税金额
     */
    private BigDecimal excludingTax;

    /**
     * 不含税金额
     */
    private BigDecimal tax;

    /**
     * 购买方名称
     */
    private String buyerName;

    /**
     * 购买方税号
     */
    private String buyerTaxId;

    /**
     * 销售方名称
     */
    private String sellerName;

    /**
     * 销售方税号
     */
    private String sellerTaxId;

    /**
     * 备注
     */
    private String remark;

    /**************** 交通 ****************/

    /**
     * 乘客姓名
     */
    private String passengerName;

    /**
     * 座位类型
     */
    private String seatType;

    /**
     * 出发地
     */
    private String departurePort;

    /**
     * 到达地
     */
    private String arrivePort;

    /**
     * 出发时间 [yyyy-MM-dd HH:mm]
     */
    private String departureTime;

    /**
     * 车次编号/航班号
     */
    private String trainNo;

    /**
     * 保险费
     */
    private BigDecimal insuranceCosts;

    /**
     * 燃油附加费
     */
    private BigDecimal fuelCosts;

    /**
     * 民航发展基金
     */
    private BigDecimal constructionCosts;

    /**************** 格式化 ****************/

    /**
     * 实例Map, 服务宜搭组件映射 [明细组件获取表头, 通过名称转换字段]
     *
     * @implSpec const headers = this.$("tableField_liv5f4d2").props.children[0].props.children.map(({ props: { label, fieldId } }) => ({ label, compId: fieldId }))
     */
    public static Map formatDtoLabelAndProp() {

        Map data = UtilMap.map("发票名称, 发票类型, 发票代码, 发票号码, 开票日期, 校验码, 价税合计, 不含税金额, 税额", "name, kindName, code, serial, date, checkCode, amount, excludingTax, tax");
        data.putAll(UtilMap.map("购买方名称, 购买方税号, 销售方名称, 销售方税号", "buyerName, buyerTaxId, sellerName, sellerTaxId"));
        data.putAll(UtilMap.map("乘客姓名, 座位类型, 出发地, 到达地, 出发时间, 车次编号/航班号, 保险费, 燃油附加费, 民航发展基金", "passengerName, seatType, departurePort, arrivePort, departureTime, trainNo, insuranceCosts, fuelCosts, constructionCosts"));
        return data;
    }

    /**
     * 格式化返回
     */
    public static Map formatResponse(Object result) {
        return UtilMap.map("result, dto", result, formatDtoLabelAndProp());
    }

    /**************** 返回值 ****************/

    // 成功状态标记
    private final static String SUC_CODE = "OK";

    /**
     * 断言错误信息
     */
    public static void assertSuccess(Map result, String kind) {
        String code = UtilMap.getString(result, "Code");
        McException.assertException(!SUC_CODE.equals(code), McREnum.VENDOR_ERROR.getCode(), kind, "tencent");
    }
}

