package com.malk.server.xbongbong;


import com.malk.utils.UtilMap;
import lombok.Data;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.stereotype.Component;

import java.util.Arrays;
import java.util.Map;

/**
 * 读取配置文件参考FilePah
 */
@Data
@Component
@ConfigurationProperties(prefix = "xbongbong")
public class XBBConf {

    /**
     * 公司corpid。接口基础参数，接口请求必传
     */
    private String corpid;


    /**
     * 访问接口的token,该值相当于密钥
     * 管理员账号登录销帮帮WEB版后台后，访问https://pfweb.xbongbong.com/#/apiToken/index查看
     */
    private String token;

    /**
     * 接口操作人userId,接口基础参数。不传默认取超管角色
     */
    private String userId;

    /**
     * 回调解析token
     */
    private String callbackToken;

    /**
     * 获取查询条件
     *
     * @implSpec 查询时间条件为秒级; 查询条件值为集合类型, 若是区间通过指定range, 值依然是集合.
     * @apiNote https://profapi.xbongbong.com/#/params?anchor=%E5%85%A5%E5%8F%82%E8%AF%B4%E6%98%8E
     */
    public static final Map getConditionMap(String attr, String symbol, Object... values) {
        return UtilMap.map("attr, symbol, value", attr, symbol, Arrays.asList(values));
    }

    /// API
    public static final String API_LIST_customer = "https://proapi.xbongbong.com/pro/v2/api/customer/list"; // 客户列表
    public static final String API_LIST_contract = "https://proapi.xbongbong.com/pro/v2/api/contract/list"; // 合同列表
    public static final String API_LIST_communicate = "https://proapi.xbongbong.com/pro/v2/api/communicate/list"; // 跟进记录
    public static final String API_LIST_opportunity = "https://proapi.xbongbong.com/pro/v2/api/opportunity/list"; // 销售机会
    public static final String API_LIST_paymentSheet = "https://proapi.xbongbong.com/pro/v2/api/paymentSheet/list"; // 回款单

}
